"""
    This file is part of the Microstepper project
    https://www.ohwr.org/project/microstepper
    
    This simple script computes microstepper output frequency
    given DDS frequency tuning words.
    
    released under GPLv3 or later license.
    Anders Wallin, 2019-2020
"""
import bigfloat
import numpy

def ftw_dds(f_hz):
    """ return AD9912 1GHz SYSCLK FTW
    """
    return numpy.round(f_hz*ftw_length/dds_clk,0)

def hz_dds(ftw):
    """ return f_out [Hz]
    """
    return dds_clk*ftw/ftw_length

def y_out(ftw1, ftw2):
    """ fractional frequency of microstepper
        F_out / F_REF
    """
    with bigfloat.precision(200):
        F1term = bigfloat.BigFloat.exact(ftw1 )
        F2term = bigfloat.BigFloat.exact(ftw2/1024.0)
        Fsum = F1term+F2term
        y = 4.0*ftw_length* bigfloat.pow( 10.0*(Fsum), -1.0)-1
        return y

dds_clk=1.0e9
ftw_length = pow(2,48)
IF = 100e6/1024.0 # 97656.25 Hz

# nominally we get zero output frequency with these 
# tuning-wrods:
F1 = ftw_dds(400e6-4*IF)    # DDS1
F2 = ftw_dds(400e6)-205         # DDS2


print 40*"-"
print "uStep output fractional frequency y"
print "FTW2 = %d (constant)" % F2
print "FTW1\t\tdFTW1\ty"
ys =[]
for delta in [-5,-4,-3,-2,-1,0,1,2,3,4,5]:
    delta=2*delta
    F1x = F1+delta
    y1 = y_out(F1x,F2) 
    ys.append(y1)
    print "%d\t%d\t%.3g" % ( F1x, delta, y1)
print "y change due to 2 steps in FTW1: %.2g" % (min(numpy.diff(ys)))
print 40*"-"
print "uStep output fractional frequency y"
print "FTW1 = %d (constant)" % F1
print "FTW2\t\tdFTW2\ty"
ys =[]
for delta in [-5,-4,-3,-2,-1,0,1,2,3,4,5]:
    delta=2*delta
    F2x = F2+delta
    y1 = y_out(F1,F2x) 
    ys.append(y1)
    print "%d\t%d\t%.3g" % ( F2x, delta, y1)

print "y change due to 2 steps in FTW2: %.2g" % (min(numpy.diff(ys)))
print 40*"-"

